"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const shared_1 = require("../shared");
class Builder {
    constructor(options, fs, logger) {
        this.options = options;
        this.fs = fs;
        this.logger = logger;
    }
    readSrcPackage() {
        return __awaiter(this, void 0, void 0, function* () {
            const { packagePath } = this.options;
            if (!(yield this.fs.fileExists(packagePath))) {
                throw new Error(`package.json not found at ${packagePath}`);
            }
            const result = yield this.fs.readJSON(packagePath);
            if (!result || typeof result !== "object") {
                throw new Error(`invalid package.json at ${packagePath}`);
            }
            return result;
        });
    }
    readConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            const { configPath } = this.options;
            let result;
            if (!this.srcPackage.tempack) {
                if (!(yield this.fs.fileExists(configPath))) {
                    throw new Error(`config not found at ${configPath}`);
                }
                result = yield this.fs.readJSON(configPath);
            }
            else {
                result = this.srcPackage.tempack;
                delete this.srcPackage.tempack;
            }
            result = Object.assign({ mergePackageWith: {}, omitPackageKeys: [], copyFiles: [], copyDirs: [] }, result);
            if (!result || typeof this.srcPackage !== "object") {
                throw new Error("Invalid config");
            }
            if (!result.mergePackageWith || typeof result.mergePackageWith !== "object") {
                throw new Error("invalid config.mergePackageWith value");
            }
            if (!Array.isArray(result.omitPackageKeys)) {
                throw new Error("invalid config.omitPackageKeys value");
            }
            if (!Array.isArray(result.copyDirs)) {
                throw new Error("invalid config.copyDirs value");
            }
            return result;
        });
    }
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            this.srcPackage = yield this.readSrcPackage();
            this.config = yield this.readConfig();
            yield this.buildAndSavePackage();
            yield this.copyFiles();
            yield this.copyDirs();
        });
    }
    buildAndSavePackage() {
        return __awaiter(this, void 0, void 0, function* () {
            const { distPath } = this.options;
            const { mergePackageWith, omitPackageKeys } = this.config;
            const distFilePath = path_1.resolve(distPath, shared_1.PACKAGE_FILE_NAME);
            let distPackage = Object.assign({}, this.srcPackage);
            for (const key of omitPackageKeys) {
                delete distPackage[key];
            }
            distPackage = Object.assign({}, distPackage, mergePackageWith);
            yield this.fs.writeJSON(distFilePath, distPackage);
            this.logger.info(`package.json saved at ${distFilePath}`);
        });
    }
    copyFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            const { srcPath, distPath } = this.options;
            const { copyFiles: fileNames } = this.config;
            for (const fileName of fileNames) {
                if (fileName &&
                    typeof fileName === "string") {
                    const srcFilePath = path_1.resolve(srcPath, fileName);
                    const distFilePath = path_1.resolve(distPath, fileName);
                    if (!(yield this.fs.fileExists(srcFilePath))) {
                        throw new Error(`file ${fileName} doesn't exists`);
                    }
                    if (yield this.fs.fileExists(distFilePath)) {
                        throw new Error(`file ${fileName} already copied`);
                    }
                    yield this.fs.copyPath(srcFilePath, distFilePath);
                    this.logger.info(`${fileName} saved at ${distFilePath}`);
                }
            }
        });
    }
    copyDirs() {
        return __awaiter(this, void 0, void 0, function* () {
            const { srcPath, distPath } = this.options;
            const { copyDirs: dirNames } = this.config;
            for (const dirName of dirNames) {
                if (dirName &&
                    typeof dirName === "string") {
                    const srcDirPath = path_1.resolve(srcPath, dirName);
                    const distDirPath = path_1.resolve(distPath, dirName);
                    if (!(yield this.fs.dirExists(srcDirPath))) {
                        throw new Error(`dir ${srcDirPath} doesn't exists`);
                    }
                    if (yield this.fs.dirExists(distDirPath)) {
                        throw new Error(`dir ${distDirPath} already copied`);
                    }
                    yield this.fs.copyPath(srcDirPath, distDirPath);
                    this.logger.info(`${dirName} saved at ${distDirPath}`);
                }
            }
        });
    }
}
exports.Builder = Builder;
//# sourceMappingURL=Builder.js.map